/* *****************************************************************************
 *
 * Filename: CalendarBaseCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  21 December 2002, 13:53
 *
 * Purpose:  Customizer for all Calendar beans (Table, List, etc.).
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.Color;
import java.awt.Font;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Calendar;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class CalendarBaseCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );

    // State
    private CalendarBase _object;
    
    // Cached
    private PropertyChangeSupport _pcsupport = null;
    
    
    /** Creates new form CalendarBaseCustomizer */
    public CalendarBaseCustomizer()
    {
        initComponents();
        
        for( int n = Calendar.SUNDAY; n < Constants.longdays.length; n++ )
            firstDayCmb.addItem( Constants.longdays[n] );

        // Accessibility stuff
        firstDayCmb.getAccessibleContext().setAccessibleDescription( resources.getString( "First_Day_Tooltip" ) );
        
        Utilities.setAccessibleLabelNames( this );
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        colorPnl = new javax.swing.JPanel();
        cellBorderColorLbl = new javax.swing.JLabel();
        cellBorderColorEdit = new com.tsac.calmaker.beans.ColorEditor();
        cellTextColorLbl = new javax.swing.JLabel();
        cellTextColorEdit = new com.tsac.calmaker.beans.ColorEditor();
        cellWendTextColorLbl = new javax.swing.JLabel();
        cellWendTextColorEdit = new com.tsac.calmaker.beans.ColorEditor();
        fontPnl = new javax.swing.JPanel();
        dayFontLbl = new javax.swing.JLabel();
        dayFontEdit = new com.tsac.calmaker.beans.FontEditor();
        dateFontLbl = new javax.swing.JLabel();
        dateFontEdit = new com.tsac.calmaker.beans.FontEditor();
        startOfWeekPnl = new javax.swing.JPanel();
        firstDayLbl = new javax.swing.JLabel();
        firstDayCmb = new javax.swing.JComboBox();
        diaryPnl = new javax.swing.JPanel();
        diaryEnabledChk = new javax.swing.JCheckBox();
        diaryFontLbl = new javax.swing.JLabel();
        diaryFontEdit = new com.tsac.calmaker.beans.FontEditor();

        setLayout(new java.awt.GridBagLayout());

        colorPnl.setLayout(new java.awt.GridBagLayout());

        colorPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Colors")));
        cellBorderColorLbl.setText(resources.getString("Cell_Border_"));
        cellBorderColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Border_Mnemonic" ).charAt(0));
        cellBorderColorLbl.setLabelFor(cellBorderColorEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 2, 2);
        colorPnl.add(cellBorderColorLbl, gridBagConstraints);

        cellBorderColorEdit.setToolTipText(resources.getString("Cell_Border_Tooltip"));
        cellBorderColorEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellBorderColorEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 2, 4);
        colorPnl.add(cellBorderColorEdit, gridBagConstraints);

        cellTextColorLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Cell_Text_"));
        cellTextColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Text_Mnemonic" ).charAt(0));
        cellTextColorLbl.setLabelFor(cellTextColorEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        colorPnl.add(cellTextColorLbl, gridBagConstraints);

        cellTextColorEdit.setToolTipText(resources.getString("Cell_Text_Tooltip"));
        cellTextColorEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellTextColorEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 4);
        colorPnl.add(cellTextColorEdit, gridBagConstraints);

        cellWendTextColorLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Cell_Text_Wend_"));
        cellWendTextColorLbl.setDisplayedMnemonic(resources.getString( "Cell_Text_Wend_Mnemonic" ).charAt(0));
        cellWendTextColorLbl.setLabelFor(cellWendTextColorEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        colorPnl.add(cellWendTextColorLbl, gridBagConstraints);

        cellWendTextColorEdit.setToolTipText(resources.getString("Cell_Text_Wend_Tooltip"));
        cellWendTextColorEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cellWendTextColorEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        colorPnl.add(cellWendTextColorEdit, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 0, 4);
        add(colorPnl, gridBagConstraints);

        fontPnl.setLayout(new java.awt.GridBagLayout());

        fontPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Fonts")));
        dayFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Day_"));
        dayFontLbl.setDisplayedMnemonic(resources.getString( "Day_Mnemonic" ).charAt(0));
        dayFontLbl.setLabelFor(dayFontEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 2, 2);
        fontPnl.add(dayFontLbl, gridBagConstraints);

        dayFontEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dayFontEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 2, 4);
        fontPnl.add(dayFontEdit, gridBagConstraints);

        dateFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Date_"));
        dateFontLbl.setDisplayedMnemonic(resources.getString( "Date_Mnemonic" ).charAt(0));
        dateFontLbl.setLabelFor(dateFontEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 4, 2);
        fontPnl.add(dateFontLbl, gridBagConstraints);

        dateFontEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dateFontEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        fontPnl.add(dateFontEdit, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        add(fontPnl, gridBagConstraints);

        startOfWeekPnl.setLayout(new java.awt.GridBagLayout());

        startOfWeekPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), resources.getString("Start_Of_Week")));
        firstDayLbl.setText(resources.getString("First_Day_"));
        firstDayLbl.setDisplayedMnemonic(resources.getString( "First_Day_Mnemonic" ).charAt(0));
        firstDayLbl.setLabelFor(firstDayCmb);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 2, 2);
        startOfWeekPnl.add(firstDayLbl, gridBagConstraints);

        firstDayCmb.setToolTipText(resources.getString("First_Day_Tooltip"));
        firstDayCmb.setMinimumSize(new java.awt.Dimension(110, 25));
        firstDayCmb.setPreferredSize(new java.awt.Dimension(110, 25));
        firstDayCmb.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                firstDayCmbActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 2, 4);
        startOfWeekPnl.add(firstDayCmb, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 0, 4);
        add(startOfWeekPnl, gridBagConstraints);

        diaryPnl.setLayout(new java.awt.GridBagLayout());

        diaryPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), resources.getString("Diary_Entries")));
        diaryEnabledChk.setMnemonic(resources.getString( "Diary_Entries_Enabled_Mnemonic" ).charAt(0));
        diaryEnabledChk.setText(resources.getString("Diary_Entries_Enabled"));
        diaryEnabledChk.setToolTipText(resources.getString("Diary_Entries_Enabled_Tooltip"));
        diaryEnabledChk.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                diaryEnabledChkActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 2, 4);
        diaryPnl.add(diaryEnabledChk, gridBagConstraints);

        diaryFontLbl.setText(java.util.ResourceBundle.getBundle("com/tsac/calmaker/beans/CalendarBeans").getString("Diary_Font"));
        diaryFontLbl.setDisplayedMnemonic(resources.getString( "Diary_Font_Mnemonic" ).charAt(0));
        diaryFontLbl.setLabelFor(diaryFontEdit);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 2);
        diaryPnl.add(diaryFontLbl, gridBagConstraints);

        diaryFontEdit.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                diaryFontEditActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 4, 4);
        diaryPnl.add(diaryFontEdit, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 4, 4);
        add(diaryPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void firstDayCmbActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_firstDayCmbActionPerformed
    {//GEN-HEADEREND:event_firstDayCmbActionPerformed
        actionFirstDayChanged();
    }//GEN-LAST:event_firstDayCmbActionPerformed

    private void cellBorderColorEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellBorderColorEditActionPerformed
    {//GEN-HEADEREND:event_cellBorderColorEditActionPerformed
        actionCellBorderColorChanged();
    }//GEN-LAST:event_cellBorderColorEditActionPerformed

    private void cellTextColorEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellTextColorEditActionPerformed
    {//GEN-HEADEREND:event_cellTextColorEditActionPerformed
        actionCellTextColorChanged();
    }//GEN-LAST:event_cellTextColorEditActionPerformed

    private void cellWendTextColorEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cellWendTextColorEditActionPerformed
    {//GEN-HEADEREND:event_cellWendTextColorEditActionPerformed
        actionCellWendTextColorChanged();
    }//GEN-LAST:event_cellWendTextColorEditActionPerformed

    private void dayFontEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dayFontEditActionPerformed
    {//GEN-HEADEREND:event_dayFontEditActionPerformed
        actionDayFontChanged();
    }//GEN-LAST:event_dayFontEditActionPerformed

    private void dateFontEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dateFontEditActionPerformed
    {//GEN-HEADEREND:event_dateFontEditActionPerformed
        actionDateFontChanged();
    }//GEN-LAST:event_dateFontEditActionPerformed

    private void diaryFontEditActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_diaryFontEditActionPerformed
    {//GEN-HEADEREND:event_diaryFontEditActionPerformed
        actionDiaryFontChanged();
    }//GEN-LAST:event_diaryFontEditActionPerformed

    private void diaryEnabledChkActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_diaryEnabledChkActionPerformed
    {//GEN-HEADEREND:event_diaryEnabledChkActionPerformed
        actionDiaryEnabledChanged();
    }//GEN-LAST:event_diaryEnabledChkActionPerformed

    public void setObject(Object obj)
    {
        if( obj instanceof CalendarBase )
        {
            CalendarBase cb = (CalendarBase) obj;
            _object = cb;
            _pcsupport = new PropertyChangeSupport( cb );

            updateCellBorderColor( cb.getCellBorderColor() );
            cellTextColorEdit.setValue( cb.getCellTextColor() );
            cellWendTextColorEdit.setValue( cb.getCellWeekendTextColor() );
            dayFontEdit.setValue( cb.getDayFont() );
            dateFontEdit.setValue( cb.getDateFont() );
            updateFirstDay( cb.getStartOfWeek() );
            updateDiaryEnabled( cb.isDiaryEnabled() );
            diaryFontEdit.setValue( cb.getDiaryFont() );
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private com.tsac.calmaker.beans.ColorEditor cellBorderColorEdit;
    private com.tsac.calmaker.beans.ColorEditor cellWendTextColorEdit;
    private com.tsac.calmaker.beans.FontEditor dayFontEdit;
    private javax.swing.JPanel diaryPnl;
    private javax.swing.JLabel dayFontLbl;
    private javax.swing.JLabel diaryFontLbl;
    private javax.swing.JLabel cellTextColorLbl;
    private javax.swing.JLabel cellBorderColorLbl;
    private javax.swing.JPanel fontPnl;
    private com.tsac.calmaker.beans.ColorEditor cellTextColorEdit;
    private com.tsac.calmaker.beans.FontEditor diaryFontEdit;
    private javax.swing.JLabel dateFontLbl;
    private javax.swing.JComboBox firstDayCmb;
    private javax.swing.JPanel colorPnl;
    private javax.swing.JLabel firstDayLbl;
    private javax.swing.JCheckBox diaryEnabledChk;
    private com.tsac.calmaker.beans.FontEditor dateFontEdit;
    private javax.swing.JLabel cellWendTextColorLbl;
    private javax.swing.JPanel startOfWeekPnl;
    // End of variables declaration//GEN-END:variables

    
    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }
    
    
    protected void updateCellBorderColor( Color c )
    {
        // Update the displayed colour
        cellBorderColorEdit.setValue( c );
        
        // Enable according to whether cell border is supported by the calendar sub-type
        boolean supported = _object.isCellBorderSupported();
        cellBorderColorLbl.setEnabled( supported );
        cellBorderColorEdit.setEnabled( supported );
    }
    
    protected void actionCellBorderColorChanged()
    {
        Color newColor = cellBorderColorEdit.getValue();
        Color oldColor = _object.getCellBorderColor();

        _object.setCellBorderColor( newColor );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "cellBorderColor", oldColor, newColor );            
    }

        
    protected void actionCellTextColorChanged()
    {
        Color newColor = cellTextColorEdit.getValue();
        Color oldColor = _object.getCellTextColor();
            
        _object.setCellTextColor( newColor );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "cellTextColor", oldColor, newColor );            
    }

        
    protected void actionCellWendTextColorChanged()
    {
        Color newColor = cellWendTextColorEdit.getValue();
        Color oldColor = _object.getCellWeekendTextColor();

        _object.setCellWeekendTextColor( newColor );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "cellWeekendTextColor", oldColor, newColor );            
    }

    
    protected void actionDayFontChanged()
    {
        Font newFont = dayFontEdit.getValue();
        Font oldFont = _object.getDayFont();

        _object.setDayFont( newFont );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "dayFont", oldFont, newFont );
    }
    
    
    protected void actionDateFontChanged()
    {
        Font newFont = dateFontEdit.getValue();
        Font oldFont = _object.getDateFont();

        _object.setDateFont( newFont );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "dateFont", oldFont, newFont );
    }
    
    
    protected void updateFirstDay( int day )
    {
        // Update the first day combo
        firstDayCmb.setSelectedIndex( day - Calendar.SUNDAY );
        
        // Disable first day combo and label if not supported
        boolean supported = _object.isStartOfWeekSupported();
        startOfWeekPnl.setEnabled( supported );
        firstDayLbl.setEnabled( supported );
        firstDayCmb.setEnabled( supported );
    }
    
    protected void actionFirstDayChanged()
    {
        // Fixes problem whereby addItem() generates an ActionEvent in constructor
        if( _object != null )
        {
            int newDay = firstDayCmb.getSelectedIndex() + Calendar.SUNDAY;
            int oldDay = _object.getStartOfWeek();

            _object.setStartOfWeek( newDay );

            if( _pcsupport != null )
                _pcsupport.firePropertyChange( "startOfWeek", oldDay, newDay );
        }
    }
    
    
    protected void actionDiaryFontChanged()
    {
        Font newFont = diaryFontEdit.getValue();
        Font oldFont = _object.getDiaryFont();

        _object.setDiaryFont( newFont );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "diaryFont", oldFont, newFont );
    }
    
    
    // Enable according to whether diary is enabled and/or supported by the calendar sub-type
    protected void enableDiaryFonts( boolean enabled )
    {
        boolean supported = _object.isDiarySupported();
        
        // Not really our responsibility, but done here for convenience
        diaryPnl.setEnabled( supported );
        diaryEnabledChk.setEnabled( supported );
        
        // Enabled diary font components if diary entries are both supported and enabled
        diaryFontLbl.setEnabled( supported && enabled );
        diaryFontEdit.setEnabled( supported && enabled );
    }
    
    
    protected void updateDiaryEnabled( boolean enabled )
    {
        diaryEnabledChk.setSelected( enabled );
        enableDiaryFonts( enabled );
    }
    
    
    protected void actionDiaryEnabledChanged()
    {
        boolean newState = diaryEnabledChk.isSelected();
        boolean oldState = _object.isDiaryEnabled();
        
        _object.setDiaryEnabled( newState );
        enableDiaryFonts( newState );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "diaryEnabled", oldState, newState );
    }
}
