/* *****************************************************************************
 *
 * Filename: FillEditor.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  30 January 2003
 *
 * Purpose:  Simple bean for selecting an fill for GridBadConstraints.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.event.*;
import java.awt.GridBagConstraints;
import java.util.ResourceBundle;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.KeyStroke;
import javax.swing.JToggleButton;
import javax.swing.event.EventListenerList;

/**
 *
 * @author  Robert
 */
public class FillEditor
    extends EditorBase
{
    // Statics
    private static ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Properties
    private int _value = -1;
    
    
    /** Creates new form BeanForm */
    public FillEditor()
    {
        initComponents();
        
        noneBtn.addActionListener( new FillAction( GridBagConstraints.NONE ) );
        horizBtn.addActionListener( new FillAction( GridBagConstraints.HORIZONTAL ) );
        vertBtn.addActionListener( new FillAction( GridBagConstraints.VERTICAL ) );        
        bothBtn.addActionListener( new FillAction( GridBagConstraints.BOTH ) );

        // Create actions for moving focus round the grid
        ActionMap actions = this.getActionMap();
        actions.put( "navigateLeft",  new GridNavigateAction( this, -1, 0 ) );
        actions.put( "navigateRight", new GridNavigateAction( this, +1, 0 ) );
        
        // Bind actions to cursor-keys 
        InputMap input = this.getInputMap( WHEN_ANCESTOR_OF_FOCUSED_COMPONENT );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0), "navigateLeft" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0), "navigateRight" );
        
        // Add a FocusListener to move the initial focus onto the selected button
        this.addFocusListener( new FocusAdapter() {
            public void focusGained( FocusEvent e ) {
                JToggleButton button = buttonFromValue( _value );
                if( button != null )
                    button.requestFocus();
            }
        } );
        
        // Set accessible names on all the buttons
        noneBtn.getAccessibleContext().setAccessibleName( resources.getString( "Fill_None_Name" ) );
        horizBtn.getAccessibleContext().setAccessibleName( resources.getString( "Fill_Horiz_Name" ) );
        vertBtn.getAccessibleContext().setAccessibleName( resources.getString( "Fill_Vert_Name" ) );
        bothBtn.getAccessibleContext().setAccessibleName( resources.getString( "Fill_Both_Name" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        fillBtnGrp = new javax.swing.ButtonGroup();
        noneBtn = new javax.swing.JToggleButton();
        horizBtn = new javax.swing.JToggleButton();
        vertBtn = new javax.swing.JToggleButton();
        bothBtn = new javax.swing.JToggleButton();

        setLayout(new java.awt.GridLayout(1, 4, 1, 1));

        noneBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/none.gif")));
        noneBtn.setToolTipText(resources.getString("Fill_None_Tooltip"));
        fillBtnGrp.add(noneBtn);
        noneBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(noneBtn);

        horizBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/horiz.gif")));
        horizBtn.setToolTipText(resources.getString("Fill_Horiz_Tooltip"));
        fillBtnGrp.add(horizBtn);
        horizBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(horizBtn);

        vertBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/vert.gif")));
        vertBtn.setToolTipText(resources.getString("Fill_Vert_Tooltip"));
        fillBtnGrp.add(vertBtn);
        vertBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(vertBtn);

        bothBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/both.gif")));
        bothBtn.setToolTipText(resources.getString("Fill_Both_Tooltip"));
        fillBtnGrp.add(bothBtn);
        bothBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(bothBtn);

    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToggleButton vertBtn;
    private javax.swing.JToggleButton bothBtn;
    private javax.swing.ButtonGroup fillBtnGrp;
    private javax.swing.JToggleButton noneBtn;
    private javax.swing.JToggleButton horizBtn;
    // End of variables declaration//GEN-END:variables

    
    public void setValue( int value )
    {
        int oldval = _value;
        
        // De-bounce - otherwise we'll loop indefinitely
        if( value != oldval )
        {
            _value = value;
            
            JToggleButton button = buttonFromValue( value );
            if( button != null )
                button.setSelected( true );
            
            firePropertyChange( "value", oldval, value );
        }
    }
    
    public int getValue()  { return _value; }
    
    
    /**
     * Re-usable code to find the appropriate button from a fill constant.
     * Used in selecting and focusing buttons.  Will return null if an invalid
     * value is passed.  There is no need to map the other way as the
     * FillAction instances take care of it.
     */
    protected JToggleButton buttonFromValue( int value )
    {
        switch( value )
        {
        case GridBagConstraints.NONE:       return noneBtn;
        case GridBagConstraints.HORIZONTAL: return horizBtn;
        case GridBagConstraints.VERTICAL:   return vertBtn;
        case GridBagConstraints.BOTH:       return bothBtn;
        }
        
        return null;
    }

    
    /**
     * Simple inner class to handle clicks on the buttons.  The value passed to
     * the constructor is passed to setValue().
     */
    private class FillAction
        implements ActionListener
    {
        private int _fill;
        
        public FillAction( int fill )
        {
            _fill = fill;
        }
        
        public void actionPerformed( ActionEvent e )
        {
            setValue( _fill );
            fireActionEvent( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, Integer.toString(_value) ) );
        }
    }
}
