/* *****************************************************************************
 *
 * Filename: FontEditor.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  03 February 2003
 *
 * Purpose:  Simple bean for picking a font.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;
import javax.swing.JToggleButton;
import javax.swing.event.EventListenerList;

/**
 *
 * @author  Robert
 */
public class FontEditor
    extends EditorBase
    implements ActionListener
{
    // Statics
    private static ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    private static final int[] sizes = { 8, 10, 12, 14, 16, 18, 20, 24, 28, 32, 36 };
    
    // Properties
    private Font _value = null;
    
    // State
    private transient boolean debounce = false;

    
    /** Creates new form BeanForm */
    public FontEditor()
    {
        initComponents();
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        String[] fonts = ge.getAvailableFontFamilyNames();
        for( int n = 0; n < fonts.length; n ++ )
        {
            Font temp = new Font( fonts[n], Font.PLAIN, 12 );
            familyCmb.addItem( temp.getFamily() );
        }
                
        for( int n = 0; n < sizes.length; n++ )
            sizeCmb.addItem( Integer.toString( sizes[n] ) );

        familyCmb.addActionListener( this );
        sizeCmb.addActionListener( this );
        boldBtn.addActionListener( this );
        italicBtn.addActionListener( this );
        
        // Add a FocusListener to move the initial focus onto the selected button
        this.addFocusListener( new FocusAdapter() {
            public void focusGained( FocusEvent e ) {
                familyCmb.requestFocus();
            }
        } );
        
        // Set accessible names on all the buttons
        familyCmb.getAccessibleContext().setAccessibleName( resources.getString( "Font_Family_Name" ) );
        familyCmb.getAccessibleContext().setAccessibleDescription( resources.getString( "Font_Family_Tooltip" ) );
        sizeCmb.getAccessibleContext().setAccessibleName( resources.getString( "Font_Size_Name" ) );
        sizeCmb.getAccessibleContext().setAccessibleDescription( resources.getString( "Font_Size_Tooltip" ) );
        boldBtn.getAccessibleContext().setAccessibleName( resources.getString( "Font_Bold_Name" ) );
        italicBtn.getAccessibleContext().setAccessibleName( resources.getString( "Font_Italic_Name" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        familyCmb = new javax.swing.JComboBox();
        sizeCmb = new javax.swing.JComboBox();
        boldBtn = new javax.swing.JToggleButton();
        italicBtn = new javax.swing.JToggleButton();

        setLayout(new java.awt.GridBagLayout());

        familyCmb.setToolTipText(resources.getString("Font_Family_Tooltip"));
        familyCmb.setMinimumSize(new java.awt.Dimension(150, 25));
        familyCmb.setPreferredSize(new java.awt.Dimension(150, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 1);
        add(familyCmb, gridBagConstraints);

        sizeCmb.setEditable(true);
        sizeCmb.setToolTipText(resources.getString("Font_Size_Tooltip"));
        sizeCmb.setMinimumSize(new java.awt.Dimension(60, 25));
        sizeCmb.setPreferredSize(new java.awt.Dimension(50, 25));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        add(sizeCmb, gridBagConstraints);

        boldBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/bold.gif")));
        boldBtn.setToolTipText(resources.getString("Font_Bold_Tooltip"));
        boldBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 1);
        add(boldBtn, gridBagConstraints);

        italicBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/italic.gif")));
        italicBtn.setToolTipText(resources.getString("Font_Italic_Tooltip"));
        italicBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(0, 1, 0, 0);
        add(italicBtn, gridBagConstraints);

    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox sizeCmb;
    private javax.swing.JComboBox familyCmb;
    private javax.swing.JToggleButton boldBtn;
    private javax.swing.JToggleButton italicBtn;
    // End of variables declaration//GEN-END:variables

    
    public void setValue( Font value )
    {
        Font oldval = _value;
        
        // De-bounce - otherwise we'll loop indefinitely
        if( value != oldval )
        {
            _value = value;
            debounce = true;
            updateFont( value );            
            debounce = false;
            firePropertyChange( "value", oldval, value );
        }
    }
    
    public Font getValue()  { return _value; }
    
    
    protected void updateFont( Font f )
    {
        familyCmb.setSelectedItem( f.getFamily() );
        boldBtn.setSelected( (f.getStyle() & Font.BOLD) != 0 );
        italicBtn.setSelected( (f.getStyle() & Font.ITALIC) != 0 );
        sizeCmb.setSelectedItem( Integer.toString(f.getSize()) );
    }

    
    public void actionPerformed( ActionEvent e )
    {
        // This prevents an infinite loop with setValue()
        if( debounce )
            return;
        
        try
        {
            int style = Font.PLAIN;
            if( boldBtn.isSelected() ) style |= Font.BOLD;
            if( italicBtn.isSelected() ) style |= Font.ITALIC;

            setValue( new Font( (String) familyCmb.getSelectedItem(), style, Integer.parseInt( (String) sizeCmb.getSelectedItem() ) ) );

            fireActionEvent( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, _value.toString() ) );
        }
        catch( NumberFormatException ex )
        {
            sizeCmb.setSelectedItem( Integer.toString(_value.getSize()) );
        }
    }
}
