/*
 * PanoramaCalendar.java
 *
 * Created on 01 February 2003, 21:30
 */

/* *****************************************************************************
 *
 * Filename: ListCalendar.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  01 February 2003
 *
 * Purpose:  Calendar bean is a thin horizontal two line table.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import java.awt.geom.*;


/**
 *
 * @author  Robert
 */
public class PanoramaCalendar
    extends CalendarBase
{
    // Cached values
    private String[] _panodays = Constants.getPanoDays( 1 );
    
    
    /** Creates new PanoramaCalendar */
    public PanoramaCalendar()
    {
        setPreferredSize( new Dimension( 400, 35 ) );
    }    
    
    
    // From CalendarBase
    public boolean isStartOfWeekSupported()  { return false; }
    
    // From CalendarBase
    public boolean isCellBorderSupported()  { return false; }

    // From CalendarBase
    public boolean isDiarySupported()  { return false; }

    
    // From CalendarBase
    protected void calclayout()
    {
        _resized = false;
        
        int width = this.getWidth();
        int height = this.getHeight();

        _cellwidth  = width / (float) 31;
        _cellheight = height / (float) 2;
        _leftmargin = _inset;            
        
        // reduce font size to fit width of a column (day names)
        String longestdaystr = Constants.getLongestString( this, _dayfont, _panodays ) + " ";
        _actualdayfont = Constants.reduceFontSizeToFit( this, _dayfont, longestdaystr, (int) _cellwidth, (int) _cellheight );
            
        _dayfontmetrics = getFontMetrics( _actualdayfont );
                        
        // reduce font size to fit width of a column (dates)
        _actualdatefont = Constants.reduceFontSizeToFit( this, _datefont, "00 ", (int) _cellwidth, (int) _cellheight );
        _datefontmetrics = getFontMetrics( _actualdatefont );
    }

    
    // From CalendarBase
    public void paint( Graphics og, double scale )
    {
        Graphics2D g = (Graphics2D) og;
        
        if( _resized  )
            calclayout();
        
        float width       = (float) (scale * this.getWidth());
        float height      = (float) (scale * this.getHeight());
        float cellwidth   = (float) (scale * _cellwidth);
        float cellheight  = (float) (scale * _cellheight);
        float leftmargin  = (float) (scale * _leftmargin);
        
        Font dayfont   = _actualdayfont.deriveFont( (float) (scale * _actualdayfont.getSize()) );
        Font datefont  = _actualdatefont.deriveFont( (float) (scale * _actualdatefont.getSize()) );
        FontMetrics dayfontmetrics   = this.getFontMetrics( dayfont );
        FontMetrics datefontmetrics  = this.getFontMetrics( datefont );

        float dayfontascent   = (float) (scale * _dayfontmetrics.getAscent());
        float datefontascent  = (float) (scale * _datefontmetrics.getAscent());
        
        Rectangle2D.Float rect;
        Line2D.Float line;
        String daystr, datestr;
        
        rect = new Rectangle2D.Float( 0, 0, width, height );
        g.setColor( Color.white );
        g.fill( rect );
        
        float hpos = leftmargin + 2;
                
        for( int col = 0; col < getDaysInMonth(); col++ )
        {
            int day = (_firstday + col) % 7;

            // Set font colour according whether weekend or not
            if( (day == 0) || (day == 6) )
                g.setColor( _cellwendtextcolor );
            else
                g.setColor( _celltextcolor );
            
            // draw day name (centered)
            g.setFont( dayfont );
            daystr = _panodays[day+1];
            g.drawString( daystr,
                hpos + (cellwidth - dayfontmetrics.stringWidth(daystr))/2,
                dayfontascent + (cellheight - dayfontascent)/2 );
            
            // draw date number (centered)                     
            g.setFont( datefont );
            datestr = "" + (col + 1);
            g.drawString( datestr,
                hpos + (cellwidth - datefontmetrics.stringWidth(datestr))/2,
                cellheight + datefontascent + (cellheight - datefontascent)/2 );
                        
            hpos += cellwidth;
        }
    }

    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff

    protected String getAccessibleName()
    {
        return resources.getString( "PanoramaCalendar_Name" );
    }
    
    protected String getAccessibleDescription()
    {
        return resources.getString( "PanoramaCalendar_Desc" );
    }    
}
