/* *****************************************************************************
 *
 * Filename: StaticImageCustomizer.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  09 February 2003
 *
 * Purpose:  Customizer for MonthImage bean.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import java.util.ResourceBundle;

/**
 *
 * @author  Robert
 */
public class StaticImageCustomizer
    extends javax.swing.JPanel
    implements java.beans.Customizer, java.beans.PropertyChangeListener
{
    // Statics
    private static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // State
    private StaticImage _object;
    
    // Cached
    private PropertyChangeSupport _pcsupport = null;
       
    
    /** Creates new customizer MonthImageCustomizer */
    public StaticImageCustomizer()
    {
        initComponents();        
        
        // Accessibility stuff
        imageTxt.getAccessibleContext().setAccessibleDescription( resources.getString( "Filename_Tooltip" ) );
        
        Utilities.setAccessibleLabelNames( this );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        java.awt.GridBagConstraints gridBagConstraints;

        statImgPnl = new javax.swing.JPanel();
        imageLbl = new javax.swing.JLabel();
        imagePnl = new javax.swing.JPanel();
        imageTxt = new javax.swing.JTextField();
        chooserBtn = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        setPreferredSize(new java.awt.Dimension(342, 250));
        statImgPnl.setLayout(new java.awt.GridBagLayout());

        statImgPnl.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(), resources.getString("Image")));
        imageLbl.setText(resources.getString("Filename_"));
        imageLbl.setDisplayedMnemonic(resources.getString( "Filename_Mnemonic" ).charAt(0));
        imageLbl.setLabelFor(imageTxt);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(0, 4, 4, 2);
        statImgPnl.add(imageLbl, gridBagConstraints);

        imagePnl.setLayout(new java.awt.GridBagLayout());

        imageTxt.setColumns(20);
        imageTxt.setToolTipText(resources.getString("Filename_Tooltip"));
        imageTxt.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                imageTxtActionPerformed(evt);
            }
        });

        imageTxt.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                imageTxtFocusLost(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        imagePnl.add(imageTxt, gridBagConstraints);

        chooserBtn.setText("...");
        chooserBtn.setToolTipText(resources.getString("Filename_Button_Tooltip"));
        chooserBtn.setMargin(new java.awt.Insets(0, 0, 0, 0));
        chooserBtn.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                chooserBtnActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        imagePnl.add(chooserBtn, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 2, 4, 4);
        statImgPnl.add(imagePnl, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        add(statImgPnl, gridBagConstraints);

    }//GEN-END:initComponents

    private void chooserBtnActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_chooserBtnActionPerformed
    {//GEN-HEADEREND:event_chooserBtnActionPerformed
        actionChooserBtnClicked();
    }//GEN-LAST:event_chooserBtnActionPerformed

    private void imageTxtFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_imageTxtFocusLost
    {//GEN-HEADEREND:event_imageTxtFocusLost
        actionImageChanged();
    }//GEN-LAST:event_imageTxtFocusLost

    private void imageTxtActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_imageTxtActionPerformed
    {//GEN-HEADEREND:event_imageTxtActionPerformed
        actionImageChanged();
    }//GEN-LAST:event_imageTxtActionPerformed

    public void setObject(Object obj)
    {
        if( obj instanceof StaticImage )
        {
            unregisterListeners();      // Unregister from previous object

            StaticImage si = (StaticImage) obj;
            _object = si;
            _pcsupport = new PropertyChangeSupport( si );
            updateImage( si.getImage() );
            registerListeners();        // Register with new object
        }
    }    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton chooserBtn;
    private javax.swing.JPanel statImgPnl;
    private javax.swing.JLabel imageLbl;
    private javax.swing.JTextField imageTxt;
    private javax.swing.JPanel imagePnl;
    // End of variables declaration//GEN-END:variables
    

    public void addPropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.addPropertyChangeListener( l );
    }

    
    public void removePropertyChangeListener( PropertyChangeListener l )
    {
        if( _pcsupport != null )
            _pcsupport.removePropertyChangeListener( l );
    }

    
    private void registerListeners()
    {
        // Listen for changes in the image list of our MonthImage
        _object.addPropertyChangeListener( StaticImage.IMAGE_PROPERTY, this );
    }
    
    
    private void unregisterListeners()
    {
        if( _object != null )
            _object.removePropertyChangeListener( StaticImage.IMAGE_PROPERTY, this );
    }
    
    
    // Implements PropertyChangeListener
    public void propertyChange( PropertyChangeEvent e )
    {
        if( (_object != null) && StaticImage.IMAGE_PROPERTY.equals( e.getPropertyName() ) )
            updateImage( _object.getImage() );
    }
    
    
    private void updateImage( String img )
    {
        imageTxt.setText( img != null ? img : "" );
    }
    
    
    private void actionImageChanged()
    {
        String oldImage = _object.getImage();
        String newImage = imageTxt.getText();
        
        if( (newImage != null) && (newImage.length() == 0) )
            newImage = null;
        
        _object.setImage( newImage );

        if( _pcsupport != null )
            _pcsupport.firePropertyChange( "image", oldImage, newImage );
    }
    
    
    private void actionChooserBtnClicked()
    {
        JFileChooser opendlg = new JFileChooser( imageTxt.getText().length() != 0 ? imageTxt.getText() : "." );
        FileFilter imgFilter;
        opendlg.addChoosableFileFilter( imgFilter = new FileFilter() {
            public boolean accept( java.io.File f ) {
                String s = f.getName().toLowerCase();
                return s.endsWith( ".jpg" ) || s.endsWith( ".jpeg" ) || s.endsWith( ".gif" ) || f.isDirectory();
            }
            public String getDescription() {
                return resources.getString("Image_files_filter");
            }
        });
        opendlg.setFileFilter( imgFilter );

        if( opendlg.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION )
        {
            imageTxt.setText( opendlg.getSelectedFile().toString() );
            actionImageChanged();
        }
    }
}
