/* *****************************************************************************
 *
 * Filename: Title.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  17 December 2002
 *
 * Purpose:  Base class for all title beans.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.*;
import javax.swing.JComponent;
import javax.accessibility.*;
import java.util.ResourceBundle;

import com.tsac.calmaker.IScaleable;
import com.tsac.calmaker.MainFrame;

public abstract class Title
    extends javax.swing.JComponent
    implements IScaleable, Accessible
{
    // Statics
    protected final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Constants
    public static final int JUSTIFY_LEFT   = 0;
    public static final int JUSTIFY_CENTER = 1;
    public static final int JUSTIFY_RIGHT  = 2;
    
    // Properties
    protected Color _color;
    protected Font _font;
    protected int _justification;
    protected String _text;
    
    
    // Default Constructor
    protected Title()
    {
        // Defaults
        _color = Color.black;
        _font  = new Font( Utilities.findFont( Utilities.SANS_SERIF ), Font.PLAIN, 24 );
        _justification = JUSTIFY_LEFT;
        setFocusable( true );
    }
    

    // Property accessors
    public Color getColor()  { return _color; }
    public void setColor( Color c )  { _color = c; repaint(); }

    public Font getFont()  { return _font; }
    
    public void setFont( Font f )
    {
        _font = f;
        setSize( getPreferredSize() );
        if( getParent() != null ) getParent().doLayout();
        repaint();
    }
    
    public int getJustification()  { return _justification; }
    public void setJustification( int j )  { _justification = j; repaint(); }
    
    public String getText()  { return _text; }

    public void setText( String t )
    {
        _text = t;
        setSize( getPreferredSize() );
        if( getParent() != null ) getParent().doLayout();
        repaint();
    }

    
    // To be implemented by concrete sub-classes
    public abstract boolean isTextSupported();
    
    // From IScaleale
    public abstract void paint( Graphics g, double scale );
    
    
    public void paint( Graphics g )
    {
        paint( g, MainFrame.scale );
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    protected abstract String getAccessibleName();
    
    protected abstract String getAccessibleDescription();
    
    
    public class AccessibleTitle
        extends JComponent.AccessibleJComponent
    {
        public String getAccessibleName()
        {
            // Use name set by the developer if set, otherwise use name from bean
            String name = super.getAccessibleName();            
            return (name != null) ? name : Title.this.getAccessibleName();
        }
        
        public String getAccessibleDescription()
        {
            // Use description set by the developer if set, otherwise use description from bean
            String name = super.getAccessibleDescription();            
            return (name != null) ? name : Title.this.getAccessibleDescription();
        }
    }
    
    
    public AccessibleContext getAccessibleContext()
    {
        if( accessibleContext == null )
            accessibleContext = new AccessibleTitle();
        
        return accessibleContext;
    }
}
